@@ -40,16 +40,6 @@
 #include "svn_repos.h"
 
 /* Define DLL export magic on Windows. */
-#ifdef WIN32
-#  ifdef SVN_SWIG_SWIGUTIL_PY_C
-#    define SVN_SWIG_SWIGUTIL_EXPORT __declspec(dllexport)
-#  else
-#    define SVN_SWIG_SWIGUTIL_EXPORT __declspec(dllimport)
-#  endif
-#else
-#  define SVN_SWIG_SWIGUTIL_EXPORT
-#endif
-
 #ifdef __cplusplus
 extern "C" {
 #endif /* __cplusplus */
@@ -57,27 +47,28 @@ extern "C" {
 
 
 /* Initialize the libsvn_swig_py library. */
-SVN_SWIG_SWIGUTIL_EXPORT
 apr_status_t svn_swig_py_initialize(void);
 
+
+/* Returns the provided python object as a FILE *object.
+ * Return the underlying FILE or NULL if the object is not a File instance.
+ */
+FILE *svn_swig_py_as_file(PyObject *pyfile);
+
 
 
 /* Functions to manage python's global interpreter lock */
-SVN_SWIG_SWIGUTIL_EXPORT
 void svn_swig_py_release_py_lock(void);
 
-SVN_SWIG_SWIGUTIL_EXPORT
 void svn_swig_py_acquire_py_lock(void);
 
 
 /*** Automatic Pool Management Functions ***/
 
 /* Set the application pool */
-SVN_SWIG_SWIGUTIL_EXPORT
 void svn_swig_py_set_application_pool(PyObject *py_pool, apr_pool_t *pool);
 
 /* Clear the application pool */
-SVN_SWIG_SWIGUTIL_EXPORT
 void svn_swig_py_clear_application_pool(void);
 
 /* Get the pool argument from the last element of tuple args.
@@ -85,7 +76,6 @@ void svn_swig_py_clear_application_pool(void);
  * subpool. Return 0 if successful. Return 1 if an error
  * occurs.
  */
-SVN_SWIG_SWIGUTIL_EXPORT
 int svn_swig_py_get_pool_arg(PyObject *args, swig_type_info *type,
     PyObject **py_pool, apr_pool_t **pool);
 
@@ -93,7 +83,6 @@ int svn_swig_py_get_pool_arg(PyObject *args, swig_type_info *type,
  * argument list. Return 0 if successful. Return 1 if an error
  * occurs.
  */
-SVN_SWIG_SWIGUTIL_EXPORT
 int svn_swig_py_get_parent_pool(PyObject *args, swig_type_info *type,
     PyObject **py_pool, apr_pool_t **pool);
 
@@ -101,54 +90,56 @@ int svn_swig_py_get_parent_pool(PyObject *args, swig_type_info *type,
 /*** SWIG Wrappers ***/
 
 /* Wrapper for SWIG_NewPointerObj */
-SVN_SWIG_SWIGUTIL_EXPORT
-PyObject *svn_swig_NewPointerObj(void *obj, swig_type_info *type,
+PyObject *svn_swig_py_new_pointer_obj(void *obj, swig_type_info *type,
                                  PyObject *pool, PyObject *args);
 
 /* Wrapper for SWIG_ConvertPtr */
-SVN_SWIG_SWIGUTIL_EXPORT
-int svn_swig_ConvertPtr(PyObject *input, void **obj, swig_type_info *type);
+int svn_swig_py_convert_ptr(PyObject *input, void **obj, swig_type_info *type);
 
 /* Wrapper for SWIG_MustGetPtr */
-SVN_SWIG_SWIGUTIL_EXPORT
-void *svn_swig_MustGetPtr(void *input, swig_type_info *type, int argnum);
+void *svn_swig_py_must_get_ptr(void *input, swig_type_info *type, int argnum);
 
 /*** Functions to expose a custom SubversionException ***/
 
 /* raise a subversion exception, created from a normal subversion
    error.  consume the error.  */
-SVN_SWIG_SWIGUTIL_EXPORT
 void svn_swig_py_svn_exception(svn_error_t *err);
 
 
 
+/* Function to get char * representation of bytes/str object. This is
+   the replacement of typemap(in, parse="s") and typemap(in, parse="z")
+   to accept both of bytes object and str object, and it assumes to be
+   used from those typemaps only.
+   Note: type of return value should be char const *, however, as SWIG
+   produces variables for C function without 'const' modifier, to avoid
+   ton of cast in SWIG produced C code we drop it from return value
+   types as well  */
+char *svn_swig_py_string_to_cstring(PyObject *input, int maybe_null,
+                                    const char * funcsym, const char * argsym);
+
 /* helper function to convert an apr_hash_t* (char* -> svnstring_t*) to
    a Python dict */
-SVN_SWIG_SWIGUTIL_EXPORT
 PyObject *svn_swig_py_prophash_to_dict(apr_hash_t *hash);
 
 /* helper function to convert an apr_hash_t* (svn_revnum_t* -> const
    char *) to a Python dict */
-SVN_SWIG_SWIGUTIL_EXPORT
 PyObject *svn_swig_py_locationhash_to_dict(apr_hash_t *hash);
 
 /* helper function to convert an apr_array_header_t* (of
    svn_merge_range_t *) to a Python list */
-SVN_SWIG_SWIGUTIL_EXPORT
-PyObject *svn_swig_py_rangelist_to_list(apr_array_header_t *rangelist,
-                                        swig_type_info *type,
-                                        PyObject *py_pool);
+PyObject *svn_swig_py_pointerlist_to_list(apr_array_header_t *list,
+                                          swig_type_info *type,
+                                          PyObject *py_pool);
 
 /* helper function to convert an apr_hash_t* (const char *->array of
    svn_merge_range_t *) to a Python dict */
-SVN_SWIG_SWIGUTIL_EXPORT
 PyObject *svn_swig_py_mergeinfo_to_dict(apr_hash_t *hash,
                                         swig_type_info *type,
                                         PyObject *py_pool);
 
 /* helper function to convert an apr_hash_t* (const char *->hash of
    mergeinfo hashes) to a Python dict */
-SVN_SWIG_SWIGUTIL_EXPORT
 PyObject *svn_swig_py_mergeinfo_catalog_to_dict(apr_hash_t *hash,
                                                 swig_type_info *type,
                                                 PyObject *py_pool);
@@ -156,83 +147,75 @@ PyObject *svn_swig_py_mergeinfo_catalog_to_dict(apr_hash_t *hash,
 /* helper function to convert an apr_hash_t *(const char *->const char
  *) to a Python dict */
 
-SVN_SWIG_SWIGUTIL_EXPORT
 PyObject *svn_swig_py_stringhash_to_dict(apr_hash_t *hash);
 
 /* convert a hash of 'const char *' -> TYPE into a Python dict */
-SVN_SWIG_SWIGUTIL_EXPORT
 PyObject *svn_swig_py_convert_hash(apr_hash_t *hash, swig_type_info *type,
                                    PyObject *py_pool);
 
 /* helper function to convert a 'char **' into a Python list of string
    objects */
-SVN_SWIG_SWIGUTIL_EXPORT
 PyObject *svn_swig_py_c_strings_to_list(char **strings);
 
 /* helper function to convert an array of 'const char *' to a Python list
    of string objects */
-SVN_SWIG_SWIGUTIL_EXPORT
 PyObject *svn_swig_py_array_to_list(const apr_array_header_t *strings);
 
 /* helper function to convert a hash mapping char * to
  * svn_log_changed_path_t * to a Python dict mapping str to str. */
-SVN_SWIG_SWIGUTIL_EXPORT
 PyObject *svn_swig_py_changed_path_hash_to_dict(apr_hash_t *hash);
 
 /* helper function to convert a hash mapping char * to
  * svn_log_changed_path2_t * to a Python dict mapping str to str. */
-SVN_SWIG_SWIGUTIL_EXPORT
 PyObject *svn_swig_py_changed_path2_hash_to_dict(apr_hash_t *hash);
 
 /* helper function to convert an array of 'svn_revnum_t' to a Python list
    of int objects */
-SVN_SWIG_SWIGUTIL_EXPORT
 PyObject *svn_swig_py_revarray_to_list(const apr_array_header_t *revs);
 
 /* helper function to convert a Python dictionary mapping strings to
    strings into an apr_hash_t mapping const char *'s to const char *'s,
    allocated in POOL. */
-SVN_SWIG_SWIGUTIL_EXPORT
 apr_hash_t *svn_swig_py_stringhash_from_dict(PyObject *dict,
                                              apr_pool_t *pool);
 
 /* helper function to convert a Python dictionary mapping strings to
    rangelists into an apr_hash_t mapping const char *'s to rangelists,
    allocated in POOL. */
-SVN_SWIG_SWIGUTIL_EXPORT
 apr_hash_t *svn_swig_py_mergeinfo_from_dict(PyObject *dict,
                                              apr_pool_t *pool);
 
 /* helper function to convert a Python dictionary mapping strings to
    strings into an 'apr_array_header_t *' of svn_prop_t *
    allocated in POOL. */
-SVN_SWIG_SWIGUTIL_EXPORT
 apr_array_header_t *svn_swig_py_proparray_from_dict(PyObject *dict,
                                                     apr_pool_t *pool);
 
 /* helper function to convert a 'apr_array_header_t *' of 'svn_prop_t
    to a Python dictionary mapping strings to strings. */
-SVN_SWIG_SWIGUTIL_EXPORT
 PyObject *svn_swig_py_proparray_to_dict(const apr_array_header_t *array);
 
+/* helper function to convert a 'apr_array_header_t *' of
+   'svn_prop_inherited_item_t' to a Python dictionary mapping strings
+   to dictionary. */
+PyObject *
+svn_swig_py_propinheriteditemarray_to_dict(const apr_array_header_t *array);
+
 /* helper function to convert a Python dictionary mapping strings to
    strings into an apr_hash_t mapping const char *'s to svn_string_t's,
    allocated in POOL. */
-SVN_SWIG_SWIGUTIL_EXPORT
 apr_hash_t *svn_swig_py_prophash_from_dict(PyObject *dict,
                                            apr_pool_t *pool);
 
 /* helper function to convert a Python dictionary mapping strings to
    integers into an apr_hash_t mapping const char *'s to revnums,
    allocated in POOL. */
-SVN_SWIG_SWIGUTIL_EXPORT
 apr_hash_t *svn_swig_py_path_revs_hash_from_dict(PyObject *dict,
                                                  apr_pool_t *pool);
 
 /* helper function to convert a Python dictionary mapping strings to
    SWIG wrappers described by type into an apr_hash_t mapping const char *'s to
    struct pointers, allocated in POOL. */
-SVN_SWIG_SWIGUTIL_EXPORT
 apr_hash_t *svn_swig_py_struct_ptr_hash_from_dict(PyObject *dict,
                                                   swig_type_info *type,
                                                   apr_pool_t *pool);
@@ -252,7 +235,6 @@ typedef int (*svn_swig_py_object_unwrap_t)(PyObject *source,
    In case of failure, raises a Python exception, presuming that seq was the
    function argument #argnum.
    pool is used to allocate the array. */
-SVN_SWIG_SWIGUTIL_EXPORT
 const apr_array_header_t *
 svn_swig_py_seq_to_array(PyObject *seq,
                          int element_size,
@@ -261,8 +243,10 @@ svn_swig_py_seq_to_array(PyObject *seq,
                          apr_pool_t *pool);
 
 /* An svn_swig_py_object_unwrap_t that extracts a char pointer from a Python
-   string. */
-SVN_SWIG_SWIGUTIL_EXPORT
+   string.
+
+   Note the lifetime of the returned string is tied to the provided Python
+   object. */
 int
 svn_swig_py_unwrap_string(PyObject *source,
                           void *destination,
@@ -270,7 +254,6 @@ svn_swig_py_unwrap_string(PyObject *source,
 
 /* An svn_swig_py_object_unwrap_t that extracts an svn_revnum_t from a Python
    integer. */
-SVN_SWIG_SWIGUTIL_EXPORT
 int
 svn_swig_py_unwrap_revnum(PyObject *source,
                           void *destination,
@@ -278,30 +261,35 @@ svn_swig_py_unwrap_revnum(PyObject *source,
 
 /* An svn_swig_py_object_unwrap_t that extracts a struct pointer from a SWIG
    wrapper. baton is expected to be a swig_type_info* describing the struct. */
-SVN_SWIG_SWIGUTIL_EXPORT
 int
 svn_swig_py_unwrap_struct_ptr(PyObject *source,
                           void *destination,
                           void *baton);
 
 /* make an editor that "thunks" from C callbacks up to Python */
-SVN_SWIG_SWIGUTIL_EXPORT
 void svn_swig_py_make_editor(const svn_delta_editor_t **editor,
-                             void **edit_baton,
-                             PyObject *py_editor,
                              apr_pool_t *pool);
 
-SVN_SWIG_SWIGUTIL_EXPORT
+/* make a parse vtable that "thunks" from C callbacks up to Python */
+void svn_swig_py_make_parse_fns3(const svn_repos_parse_fns3_t **parse_fns3,
+                                 apr_pool_t *pool);
+
 apr_file_t *svn_swig_py_make_file(PyObject *py_file,
                                   apr_pool_t *pool);
 
-SVN_SWIG_SWIGUTIL_EXPORT
 svn_stream_t *svn_swig_py_make_stream(PyObject *py_io,
                                       apr_pool_t *pool);
 
+/* Convert ops, a C array of num_ops elements, to a Python list of SWIG
+   objects with descriptor op_type_info and pool set to parent_pool. */
+PyObject *
+svn_swig_py_convert_txdelta_op_c_array(int num_ops,
+                                       svn_txdelta_op_t *ops,
+                                       swig_type_info * op_type_info,
+                                       PyObject *parent_pool);
+
 /* a notify function that executes a Python function that is passed in
    via the baton argument */
-SVN_SWIG_SWIGUTIL_EXPORT
 void svn_swig_py_notify_func(void *baton,
                              const char *path,
                              svn_wc_notify_action_t action,
@@ -311,21 +299,25 @@ void svn_swig_py_notify_func(void *baton,
                              svn_wc_notify_state_t prop_state,
                              svn_revnum_t revision);
 
-SVN_SWIG_SWIGUTIL_EXPORT
 void svn_swig_py_notify_func2(void *baton,
                               const svn_wc_notify_t *notify,
                               apr_pool_t *pool);
 
 /* a status function that executes a Python function that is passed in
    via the baton argument */
-SVN_SWIG_SWIGUTIL_EXPORT
 void svn_swig_py_status_func(void *baton,
                              const char *path,
                              svn_wc_status_t *status);
 
+/* a client status function that executes a Python function that is passed in
+   via the baton argument */
+svn_error_t *svn_swig_py_client_status_func(void *baton,
+                                            const char *path,
+                                            const svn_client_status_t *status,
+                                            apr_pool_t *scratch_pool);
+
 /* a svn_delta_path_driver callback that executes a Python function
   that is passed in via the baton argument */
-SVN_SWIG_SWIGUTIL_EXPORT
 svn_error_t *svn_swig_py_delta_path_driver_cb_func(void **dir_baton,
                                                    void *parent_baton,
                                                    void *callback_baton,
@@ -334,24 +326,27 @@ svn_error_t *svn_swig_py_delta_path_driver_cb_func(void **dir_baton,
 
 /* a status function that executes a Python function that is passed in
    via the baton argument */
-SVN_SWIG_SWIGUTIL_EXPORT
 void svn_swig_py_status_func2(void *baton,
                               const char *path,
                               svn_wc_status2_t *status);
 
 /* a cancel function that executes a Python function passed in via the
    cancel_baton argument. */
-SVN_SWIG_SWIGUTIL_EXPORT
 svn_error_t *svn_swig_py_cancel_func(void *cancel_baton);
 
 /* thunked fs get_locks function */
-SVN_SWIG_SWIGUTIL_EXPORT
 svn_error_t *svn_swig_py_fs_get_locks_func(void *baton,
                                            svn_lock_t *lock,
                                            apr_pool_t *pool);
 
+svn_error_t *svn_swig_py_fs_lock_callback(
+                    void *baton,
+                    const char *path,
+                    const svn_lock_t *lock,
+                    svn_error_t *ra_err,
+                    apr_pool_t *pool);
+
 /* thunked commit log fetcher */
-SVN_SWIG_SWIGUTIL_EXPORT
 svn_error_t *svn_swig_py_get_commit_log_func(const char **log_msg,
                                              const char **tmp_file,
                                              const apr_array_header_t *
@@ -360,7 +355,6 @@ svn_error_t *svn_swig_py_get_commit_log_func(const char **log_msg,
                                              apr_pool_t *pool);
 
 /* thunked repos authz callback function */
-SVN_SWIG_SWIGUTIL_EXPORT
 svn_error_t *svn_swig_py_repos_authz_func(svn_boolean_t *allowed,
                                           svn_fs_root_t *root,
                                           const char *path,
@@ -368,14 +362,12 @@ svn_error_t *svn_swig_py_repos_authz_func(svn_boolean_t *allowed,
                                           apr_pool_t *pool);
 
 /* thunked history callback function */
-SVN_SWIG_SWIGUTIL_EXPORT
 svn_error_t *svn_swig_py_repos_history_func(void *baton,
                                             const char *path,
                                             svn_revnum_t revision,
                                             apr_pool_t *pool);
 
 /* thunked log receiver function */
-SVN_SWIG_SWIGUTIL_EXPORT
 svn_error_t *svn_swig_py_log_receiver(void *py_receiver,
                                       apr_hash_t *changed_paths,
                                       svn_revnum_t rev,
@@ -385,27 +377,38 @@ svn_error_t *svn_swig_py_log_receiver(void *py_receiver,
                                       apr_pool_t *pool);
 
 /* thunked log receiver2 function */
-SVN_SWIG_SWIGUTIL_EXPORT
 svn_error_t *svn_swig_py_log_entry_receiver(void *baton,
                                             svn_log_entry_t *log_entry,
                                             apr_pool_t *pool);
 
+/* thunked repos freeze function */
+svn_error_t *svn_swig_py_repos_freeze_func(void *baton,
+                                           apr_pool_t *pool);
+
+/* thunked fs freeze function */
+svn_error_t *svn_swig_py_fs_freeze_func(void *baton,
+                                        apr_pool_t *pool);
+
+/* thunked proplist receiver2 function */
+svn_error_t *svn_swig_py_proplist_receiver2(void *baton,
+                                            const char *path,
+                                            apr_hash_t *prop_hash,
+                                            apr_array_header_t *inherited_props,
+                                            apr_pool_t *pool);
+
 /* thunked info receiver function */
-SVN_SWIG_SWIGUTIL_EXPORT
 svn_error_t *svn_swig_py_info_receiver_func(void *py_receiver,
                                             const char *path,
                                             const svn_info_t *info,
                                             apr_pool_t *pool);
 
 /* thunked location segments receiver function */
-SVN_SWIG_SWIGUTIL_EXPORT
 svn_error_t *
 svn_swig_py_location_segment_receiver_func(svn_location_segment_t *segment,
                                            void *baton,
                                            apr_pool_t *pool);
 
 /* thunked blame receiver function */
-SVN_SWIG_SWIGUTIL_EXPORT
 svn_error_t *svn_swig_py_client_blame_receiver_func(void *baton,
                                                     apr_int64_t line_no,
                                                     svn_revnum_t revision,
@@ -415,14 +418,18 @@ svn_error_t *svn_swig_py_client_blame_receiver_func(void *baton,
                                                     apr_pool_t *pool);
 
 /* thunked changelist receiver function */
-SVN_SWIG_SWIGUTIL_EXPORT
 svn_error_t *svn_swig_py_changelist_receiver_func(void *baton,
                                                   const char *path,
                                                   const char *changelist,
                                                   apr_pool_t *pool);
 
 /* auth provider callbacks */
-SVN_SWIG_SWIGUTIL_EXPORT
+svn_error_t * svn_swig_py_auth_gnome_keyring_unlock_prompt_func(
+        char **keyring_passwd,
+        const char *keyring_name,
+        void *baton,
+        apr_pool_t *pool);
+
 svn_error_t *svn_swig_py_auth_simple_prompt_func(
     svn_auth_cred_simple_t **cred,
     void *baton,
@@ -431,7 +438,6 @@ svn_error_t *svn_swig_py_auth_simple_prompt_func(
     svn_boolean_t may_save,
     apr_pool_t *pool);
 
-SVN_SWIG_SWIGUTIL_EXPORT
 svn_error_t *svn_swig_py_auth_username_prompt_func(
     svn_auth_cred_username_t **cred,
     void *baton,
@@ -439,7 +445,6 @@ svn_error_t *svn_swig_py_auth_username_prompt_func(
     svn_boolean_t may_save,
     apr_pool_t *pool);
 
-SVN_SWIG_SWIGUTIL_EXPORT
 svn_error_t *svn_swig_py_auth_ssl_server_trust_prompt_func(
     svn_auth_cred_ssl_server_trust_t **cred,
     void *baton,
@@ -449,7 +454,6 @@ svn_error_t *svn_swig_py_auth_ssl_server_trust_prompt_func(
     svn_boolean_t may_save,
     apr_pool_t *pool);
 
-SVN_SWIG_SWIGUTIL_EXPORT
 svn_error_t *svn_swig_py_auth_ssl_client_cert_prompt_func(
     svn_auth_cred_ssl_client_cert_t **cred,
     void *baton,
@@ -457,7 +461,6 @@ svn_error_t *svn_swig_py_auth_ssl_client_cert_prompt_func(
     svn_boolean_t may_save,
     apr_pool_t *pool);
 
-SVN_SWIG_SWIGUTIL_EXPORT
 svn_error_t *svn_swig_py_auth_ssl_client_cert_pw_prompt_func(
     svn_auth_cred_ssl_client_cert_pw_t **cred,
     void *baton,
@@ -465,32 +468,35 @@ svn_error_t *svn_swig_py_auth_ssl_client_cert_pw_prompt_func(
     svn_boolean_t may_save,
     apr_pool_t *pool);
 
-SVN_SWIG_SWIGUTIL_EXPORT
+/* auth cleanup callback */
+svn_error_t *svn_swig_py_config_auth_walk_func(svn_boolean_t *delete_cred,
+                                               void *walk_baton,
+                                               const char *cred_kind,
+                                               const char *realmstring,
+                                               apr_hash_t *hash,
+                                               apr_pool_t *scratch_pool);
+
 void
 svn_swig_py_setup_ra_callbacks(svn_ra_callbacks2_t **callbacks,
                                void **baton,
                                PyObject *py_callbacks,
                                apr_pool_t *pool);
 
-SVN_SWIG_SWIGUTIL_EXPORT
 svn_wc_diff_callbacks2_t *
 svn_swig_py_setup_wc_diff_callbacks2(void **baton,
                                      PyObject *py_callbacks,
                                      apr_pool_t *pool);
 
-SVN_SWIG_SWIGUTIL_EXPORT
 svn_error_t *svn_swig_py_commit_callback2(const svn_commit_info_t *commit_info,
                                           void *baton,
                                           apr_pool_t *pool);
 
-SVN_SWIG_SWIGUTIL_EXPORT
 svn_error_t *svn_swig_py_commit_callback(svn_revnum_t new_revision,
                                          const char *date,
                                          const char *author,
                                          void *baton);
 
 
-SVN_SWIG_SWIGUTIL_EXPORT
 svn_error_t *svn_swig_py_ra_file_rev_handler_func(
                     void *baton,
                     const char *path,
@@ -501,7 +507,6 @@ svn_error_t *svn_swig_py_ra_file_rev_handler_func(
                     apr_array_header_t *prop_diffs,
                     apr_pool_t *pool);
 
-SVN_SWIG_SWIGUTIL_EXPORT
 svn_error_t *svn_swig_py_ra_lock_callback(
                     void *baton,
                     const char *path,
@@ -510,19 +515,18 @@ svn_error_t *svn_swig_py_ra_lock_callback(
                     svn_error_t *ra_err,
                     apr_pool_t *pool);
 
-SVN_SWIG_SWIGUTIL_EXPORT
-extern const svn_ra_reporter2_t swig_py_ra_reporter2;
+const svn_ra_reporter2_t *svn_swig_py_get_ra_reporter2(void);
 
-/* Get a list of ops from a window. Used to replace the naive
-   svn_txdelta_window_t.ops accessor. op_type_info is supposed to be
-   the SWIG descriptor of "svn_txdelta_op_t *". window_pool is supposed
-   to be the pool associated with the window proxy and used for wrapping
-   the op objects. */
-SVN_SWIG_SWIGUTIL_EXPORT
-PyObject *
-svn_swig_py_txdelta_window_t_ops_get(svn_txdelta_window_t *window,
-                                     swig_type_info * op_type_info,
-                                     PyObject *window_pool);
+svn_boolean_t
+svn_swig_py_config_enumerator2(const char *name,
+                               const char *value,
+                               void *baton,
+                               apr_pool_t *pool);
+
+svn_boolean_t
+svn_swig_py_config_section_enumerator2(const char *name,
+                                       void *baton,
+                                       apr_pool_t *pool);
 
 #ifdef __cplusplus
 }
