@@ -167,38 +167,38 @@ def test_stats(sbox):
                       'Histogram of directory property sizes:',
                       'Histogram of directory property representation sizes:']
   patterns_to_find = {
-    'Reading revisions' : ['\s+ 0[ 0-9]*'],
-    'Global .*'         : ['.*\d+ bytes in .*\d+ revisions',
-                           '.*\d+ bytes in .*\d+ changes',
-                           '.*\d+ bytes in .*\d+ node revision records',
-                           '.*\d+ bytes in .*\d+ representations',
-                           '.*\d+ bytes expanded representation size',
-                           '.*\d+ bytes with rep-sharing off' ],
-    'Noderev .*'        : ['.*\d+ bytes in .*\d+ nodes total',
-                           '.*\d+ bytes in .*\d+ directory noderevs',
-                           '.*\d+ bytes in .*\d+ file noderevs' ],
-    'Representation .*' : ['.*\d+ bytes in .*\d+ representations total',
-                           '.*\d+ bytes in .*\d+ directory representations',
-                           '.*\d+ bytes in .*\d+ file representations',
-                           '.*\d+ bytes in .*\d+ representations of added file nodes',
-                           '.*\d+ bytes in .*\d+ directory property representations',
-                           '.*\d+ bytes in .*\d+ file property representations',
-                           '.*\d+ average delta chain length',
-                           '.*\d+ bytes in header & footer overhead' ],
+    'Reading revisions' : [r'\s+ 0[ 0-9]*'],
+    'Global .*'         : [r'.*\d+ bytes in .*\d+ revisions',
+                           r'.*\d+ bytes in .*\d+ changes',
+                           r'.*\d+ bytes in .*\d+ node revision records',
+                           r'.*\d+ bytes in .*\d+ representations',
+                           r'.*\d+ bytes expanded representation size',
+                           r'.*\d+ bytes with rep-sharing off' ],
+    'Noderev .*'        : [r'.*\d+ bytes in .*\d+ nodes total',
+                           r'.*\d+ bytes in .*\d+ directory noderevs',
+                           r'.*\d+ bytes in .*\d+ file noderevs' ],
+    'Representation .*' : [r'.*\d+ bytes in .*\d+ representations total',
+                           r'.*\d+ bytes in .*\d+ directory representations',
+                           r'.*\d+ bytes in .*\d+ file representations',
+                           r'.*\d+ bytes in .*\d+ representations of added file nodes',
+                           r'.*\d+ bytes in .*\d+ directory property representations',
+                           r'.*\d+ bytes in .*\d+ file property representations',
+                           r'.*\d+ average delta chain length',
+                           r'.*\d+ bytes in header & footer overhead' ],
     '.* representation statistics:' :
-                          ['.*\d+ bytes in .*\d+ reps',
-                           '.*\d+ bytes in .*\d+ shared reps',
-                           '.*\d+ bytes expanded size',
-                           '.*\d+ bytes expanded shared size',
-                           '.*\d+ bytes with rep-sharing off',
-                           '.*\d+ shared references',
-                           '.*\d+ average delta chain length'],
-    'Largest.*:'        : ['.*\d+ r\d+ */\S*'],
+                          [r'.*\d+ bytes in .*\d+ reps',
+                           r'.*\d+ bytes in .*\d+ shared reps',
+                           r'.*\d+ bytes expanded size',
+                           r'.*\d+ bytes expanded shared size',
+                           r'.*\d+ bytes with rep-sharing off',
+                           r'.*\d+ shared references',
+                           r'.*\d+ average delta chain length'],
+    'Largest.*:'        : [r'.*\d+ r\d+ */\S*'],
     'Extensions by number .*:' :
-                          ['.*\d+ \( ?\d+%\) representations'],
+                          [r'.*\d+ \( ?\d+%\) representations'],
     'Extensions by size .*:' :
-                          ['.*\d+ \( ?\d+%\) bytes'],
-    'Histogram of .*:'  : ['.*\d+ \.\. < \d+.*\d+ \( ?\d+%\) bytes in *\d+ \( ?\d+%\) items']
+                          [r'.*\d+ \( ?\d+%\) bytes'],
+    'Histogram of .*:'  : [r'.*\d+ \.\. < \d+.*\d+ \( ?\d+%\) bytes in *\d+ \( ?\d+%\) items']
   }
 
   # check that the output contains all sections
